#!/usr/bin/env python
#-*- coding:utf-8 –*-
#-----------------------------------------------------------------------------
# $Header: $
#-----------------------------------------------------------------------------
# Summary:            Implementation of Frequency Counter Calibration & Test
# 
# Authors:            Jensen.Cai
#
# Started:            4/20/2015
#
#Version:             V1.7R1
#
# Copyright 2013-2016 Siglent Corporation. All Rights Reserved.
#
#-----------------------------------------------------------------------------

import sys, time, math
sys.path.append(sys.path[0]+'\\Prod Lib')
from reminder import popup
from test_info import testInfo_show
from win32com_excel import *
from device_remote import *
from param import *
from file import *

#-----------------------------------------------------------------------------
class FrqCnt(FrqCntParam):
    frqCnt_delay = 5
    frqCnt_duty = 50
    def __init__(self):
        super(FrqCnt, self).__init__()
        self.frqCnt = FrqCntRemote(self.frqCnt_delay)
        self.fail_info = []
    def make_file(self):
        report = TestReport('FreqCounterCal')
        xls_name = report.create()
        self.xls = easyExcel(xls_name)
        report.head_write(self.xls)
    def cal(self):
        popup(u'注意！Notice！', u'请将通道一连接到频率计输入端\nPlease connect CH1 to counter')
        print '\nFrequency Counter Trigger Level Calibrating...'
        self.row = 8
        self.preset()
        for i in range(2):
            self.frqCnt.set_trgLvl(self.cal_trgLvl[i])
            self.frqCnt.set_ofst('C1', self.cal_trgLvl[i])
            self.adjust_trgLvl(self.cal_trgLvl[i])
            self.cal_reg[i] = self.frqCnt.get_trgLvl_reg()
            self.row += 20
        self.set_off()
        self.save_calCurve()
    def test(self):
        print 'Frequency Counter Trigger Level Testing...'
        self.row = 49
        test_times = 1
        up_limit = self.frqCnt_duty + self.test_err
        low_limit = self.frqCnt_duty - self.test_err
        self.preset()
        for amp in self.test_amp:
            self.frqCnt.set_amp('C1', amp)
            for frq in self.test_frq:
                self.frqCnt.set_frq('C1', frq)
                for ofst in self.test_ofst:
                    self.frqCnt.set_trgLvl(ofst)
                    self.frqCnt.set_ofst('C1', ofst)
                    
                    read_duty = self.frqCnt.get_duty()
                    self.xls.setCell('Sheet1', self.row + test_times, 1, '%s' % amp)
                    self.xls.setCell('Sheet1', self.row + test_times, 2, '%s' % frq)
                    self.xls.setCell('Sheet1', self.row + test_times, 3, '%s' % ofst)
                    self.xls.setCell('Sheet1', self.row + test_times, 4, '%s' % read_duty)
                    self.xls.setCell('Sheet1', self.row + test_times, 5, '%s ~ %s' % (low_limit, up_limit))
                    if read_duty > up_limit or read_duty < low_limit:
                        print 'Test: %sVpp, %sHz, TrgLvl=%sV, Read Duty = %s%% Fail' % (amp, frq, ofst, read_duty)
                        self.xls.setCell('Sheet1', self.row + test_times, 6, 'Fail')
                        self.fail_info.append(u'频率计触发电平%sV %sHz检测失败' % (ofst, frq))
                        self.fail_info.append('Frequency Counter Trigger Level Test: %sV %sHz Fail' % (ofst, frq))
                    else:
                        print 'Test: %sVpp, %sHz, TrgLvl=%sV, Read Duty = %s%% Pass' % (amp, frq, ofst, read_duty)
                        self.xls.setCell('Sheet1', self.row + test_times, 6, 'Pass')
                    test_times += 1
        self.set_off()
    def preset(self):
        self.frqCnt.set_state_on()
        self.frqCnt.set_coupMod('DC')
        self.frqCnt.set_output_load('C1','HZ')
        self.frqCnt.set_bswv('C1', 'SINE', self.cal_frq, self.cal_amp)
        self.frqCnt.set_duty('C1', self.frqCnt_duty)
        self.frqCnt.set_output_on('C1')
    def adjust_trgLvl(self, trg_lvl):
        up_duty, low_duty = self.frqCnt_duty+self.cal_err, self.frqCnt_duty-self.cal_err
        adj_duty = self.frqCnt.get_duty()
        adj_lvl = trg_lvl
        adj_times = 1
        print 'Calibrating Times:%s, DestDuty=%s, ReadDuty=%f' % (adj_times, self.frqCnt_duty, adj_duty)
        self.xls.setCell('Sheet1', self.row + adj_times, 1, '%s' % trg_lvl)
        self.xls.setCell('Sheet1', self.row + adj_times, 2, '%s' % adj_times)
        self.xls.setCell('Sheet1', self.row + adj_times, 3, '%s' % adj_lvl)
        self.xls.setCell('Sheet1', self.row + adj_times, 4, '%s' % adj_duty)
        self.xls.setCell('Sheet1', self.row + adj_times, 5, '%s ~ %s' % (low_duty, up_duty))
        while adj_duty > up_duty or adj_duty < low_duty:
            adj_lvl += self.cal_amp / 2 * math.sin((adj_duty - self.frqCnt_duty) * math.pi / 100)
            self.frqCnt.set_trgLvl(adj_lvl)
            adj_duty = self.frqCnt.get_duty()
            adj_times += 1
            print 'Calibrating Times:%s, DestDuty=%s, ReadDuty=%f' % (adj_times, self.frqCnt_duty, adj_duty)
            self.xls.setCell('Sheet1', self.row + adj_times, 2, '%s' % adj_times)
            self.xls.setCell('Sheet1', self.row + adj_times, 3, '%s' % adj_lvl)
            self.xls.setCell('Sheet1', self.row + adj_times, 4, '%s' % adj_duty)
            self.xls.setCell('Sheet1', self.row + adj_times, 5, '%s ~ %s' % (low_duty, up_duty))
            if adj_times >= 20:
                self.frqCnt.set_trgLvl(trg_lvl)
                print 'Frequency Counter Trigger Level Calibration: %sV Fail' % trg_lvl
                self.xls.setCell('Sheet1', self.row + 1, 6, 'Fail')
                self.fail_info.append(u'频率计触发电平%s校正失败' % trg_lvl)
                self.fail_info.append('Frequency Counter Trigger Level Calibration: %sV Fail\n' % trg_lvl)
                break
        if adj_times < 20:
            print 'Frequency Counter Trigger Level Calibration: %sV Pass\n' % trg_lvl
            self.xls.setCell('Sheet1', self.row + 1, 6, 'Pass')
    def save_calCurve(self):
        slope = (self.cal_reg[0] - self.cal_reg[1]) / (self.cal_trgLvl[0] - self.cal_trgLvl[1])
        delta = self.cal_reg[1]
        self.frqCnt.set_calData(round(slope, 1), round(delta, 1))
        self.frqCnt.save_adjData()
        time.sleep(self.save_time)
    def set_off(self):
        self.frqCnt.set_state_off()
        self.frqCnt.set_output_off('C1')
    def saveAndClose(self):
        self.xls.save()
        time.sleep(0.2)
        self.xls.close()
        time.sleep(0.2)
        self.frqCnt.close()
        time.sleep(0.2)
    def show_result(self):
        if self.fail_info == []:
            testInfo_show([u'频率计触发电平校正通过，请进行下一步操作', 'Trigger Level Calibration: Pass'])
        else:
            self.fail_info.insert(0, u'频率计触发电平校正失败\n')
            self.fail_info.insert(1, 'Trigger Level Calibration: Fail\n')
            testInfo_show(self.fail_info)

if __name__ == '__main__':
    foo = FrqCnt()
    foo.make_file()
    foo.cal()
    foo.test()
    foo.saveAndClose()
    foo.show_result()
